/*
 SimpleJournal: An example LiveJournal client written using LJKit.
 This program and its source code are placed in the public domain.
 */

#import "Friends.h"
#import "Groups.h"

@implementation Friends

- (void)awakeFromNib
{
    NSTableColumn *tc;
    id cell;

    // Configure the icon column to display images
    tc = [tableView tableColumnWithIdentifier:@"icon"];
    cell = [[NSImageCell alloc] init];
    [cell setImageScaling:NSScaleNone];
    [tc setDataCell:cell];
    [cell release];
    // Configure the username column to draw background
    tc = [tableView tableColumnWithIdentifier:@"username"];
    cell = [tc dataCell];
    [cell setDrawsBackground:YES];
}

- (LJFriend *)selectedFriend
{
    if ([tableView numberOfSelectedRows] != 1) return nil;
    return [friendArray objectAtIndex:[tableView selectedRow]];
}

- (LJFriend *)friendAtIndex:(int)index
{
    return [friendArray objectAtIndex:index];
}

- (void)reload
{
    int count = [groups numberOfSelectedGroups];

    [friendArray release];
    if (count == 1) {
        LJGroup *group = [groups selectedGroup];
        if (group) {
            friendArray = [[group memberArray] retain];
        } else {
            friendArray = [[[controller account] friendArray] retain];
        }
    } else {
        friendArray = nil;
    }
    [tableView reloadData];
    [self updateFriendDetails];
}

- (void)addFriendWithUsername:(NSString *)username
{
    LJFriend *friend;
    LJGroup *group = [groups selectedGroup];
    int f;

    friend = [[controller account] addFriendWithUsername:username];
    if (group) [group addFriend:friend];
    [self reload];
    // Select New Friend and Update Display
    f = [friendArray indexOfObject:friend] + 1;
    [tableView selectRow:f byExtendingSelection:NO];
    [window setDocumentEdited:YES];
}

- (void)removeSelectedFriends
{
    NSEnumerator *rowEnumerator = [tableView selectedRowEnumerator];
    NSNumber *rowNumber;
    LJGroup *group = [groups selectedGroup];
    LJFriend *friend;

    while (rowNumber = [rowEnumerator nextObject]) {
        friend = [friendArray objectAtIndex:[rowNumber intValue]];
        if (group) {
            [group removeFriend:friend];
        } else {
            [[controller account] removeFriend:friend];
        }
    }
    [self reload];
    [window setDocumentEdited:YES];
}

- (void)updateFriendDetails
{
    LJFriend *friend = [self selectedFriend];

    if (friend) {
        [fullNameField setObjectValue:[friend fullname]];
        [fullNameField setTextColor:[friend foregroundColor]];
        [fullNameField setBackgroundColor:[friend backgroundColor]];
        [fgColorWell setEnabled:YES];
        [fgColorWell setColor:[friend foregroundColor]];
        [bgColorWell setEnabled:YES];
        [bgColorWell setColor:[friend backgroundColor]];
        [birthDateField setObjectValue:[friend birthDate]];
        [statusField setObjectValue:[friend accountStatus]];
        [typeField setObjectValue:[friend accountType]];
    } else {
        [fullNameField setStringValue:([tableView numberOfSelectedRows] ?
                                       @"Multiple Selection" : @"No Selection")];
        [fullNameField setTextColor:[NSColor blackColor]];
        [fullNameField setBackgroundColor:[NSColor whiteColor]];
        [fgColorWell setEnabled:NO];
        [fgColorWell setColor:[NSColor blackColor]];
        [bgColorWell setEnabled:NO];
        [bgColorWell setColor:[NSColor whiteColor]];
        [birthDateField setObjectValue:nil];
        [statusField setObjectValue:nil];
        [typeField setObjectValue:nil];
    }
}

- (IBAction)changeBackgroundColor:(id)sender
{
    [[self selectedFriend] setBackgroundColor:[sender color]];
    [fullNameField setBackgroundColor:[sender color]];
    [window setDocumentEdited:YES];
}

- (IBAction)changeForegroundColor:(id)sender
{
    [[self selectedFriend] setForegroundColor:[sender color]];
    [fullNameField setTextColor:[sender color]];
    [window setDocumentEdited:YES];
}

- (IBAction)beginNewFriend:(id)sender
{
    [NSApp beginSheet:newFriendPanel modalForWindow:window
        modalDelegate:nil didEndSelector:nil contextInfo:NULL];
}

- (IBAction)endNewFriend:(id)sender
{
    [NSApp endSheet:newFriendPanel];
    [newFriendPanel orderOut:nil];
    if ([sender tag]) {
        [self addFriendWithUsername:[newFriendUsernameField stringValue]];
    }
}

- (IBAction)removeFriend:(id)sender
{
    [self removeSelectedFriends];
}

// NSTableDataSource:
- (int)numberOfRowsInTableView:(NSTableView *)tv
{
    return [friendArray count];
}

// NSTableDataSource:
- (id)tableView:(NSTableView *)tv objectValueForTableColumn:(NSTableColumn *)tc
            row:(int)row
{
    LJFriend *friend = [friendArray objectAtIndex:row];
    if ([[tc identifier] isEqualToString:@"username"]) {
        return [friend username];
    } else {
        if ([[friend accountType] isEqualToString:@"community"]) {
            return [NSImage imageNamed:@"communityicon"];
        } else {
            return [NSImage imageNamed:@"usericon"];
        }
    }
}

// NSTableDataSource:
- (BOOL)tableView:(NSTableView *)tv writeRows:(NSArray *)rows
     toPasteboard:(NSPasteboard *)pboard
{
    [pboard declareTypes:[NSArray arrayWithObject:@"FriendRows"] owner:self];
    [pboard setPropertyList:rows forType:@"FriendRows"];
    return YES;
}

// NSTableView Delegate:
- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
    [self updateFriendDetails];
}

- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell
   forTableColumn:(NSTableColumn *)tableColumn row:(int)rowIndex
{
    if ([[tableColumn identifier] isEqualToString:@"username"]) {
        LJFriend *friend = [friendArray objectAtIndex:rowIndex];
        [cell setTextColor:[friend foregroundColor]];
        [cell setBackgroundColor:[friend backgroundColor]];
    }
}

@end
