/*
 SimpleJournal: An example LiveJournal client written using LJKit.
 This program and its source code are placed in the public domain.
 */

#import "Groups.h"
#import "Friends.h"

@implementation Groups

- (void)awakeFromNib
{
    [tableView registerForDraggedTypes:[NSArray arrayWithObject:@"FriendRows"]];
}

- (int)numberOfSelectedGroups
{
    return [tableView numberOfSelectedRows];
}

- (LJGroup *)selectedGroup
{
    int g = [tableView selectedRow];
    if (g < 1 || g >= [groupArray count] + 1) return nil;
    return [groupArray objectAtIndex:(g - 1)];
}

- (void)reload
{
    [groupArray release];
    groupArray = [[[controller account] groupArray] retain];
    [tableView reloadData];
    [friends reload];
}

- (void)removeSelectedGroups
{
    NSEnumerator *rowEnumerator = [tableView selectedRowEnumerator];
    NSNumber *rowNumber;
    LJGroup *group;
    int g;

    while (rowNumber = [rowEnumerator nextObject]) {
        g = [rowNumber intValue] - 1;
        if (g >= 0) {
            group = [groupArray objectAtIndex:g];
            [[controller account] removeGroup:group];
        }
    }
    [self reload];
    [window setDocumentEdited:YES];
}

- (IBAction)newGroup:(id)sender
{
    LJGroup *group = [[controller account] newGroupWithName:@"New Group"];
    int g;
    [self reload];
    g = [groupArray indexOfObject:group] + 1;
    [tableView selectRow:g byExtendingSelection:NO];
    [tableView editColumn:0 row:g withEvent:nil select:YES];
}

- (IBAction)removeGroup:(id)sender
{
    if ([tableView numberOfSelectedRows] > 0) {
        [self removeSelectedGroups];
        [self reload];
    }
}

// NSTableDataSource:
- (int)numberOfRowsInTableView:(NSTableView *)tv
{
    return 1 + [groupArray count];
}

// NSTableDataSource:
- (id)tableView:(NSTableView *)tv objectValueForTableColumn:(NSTableColumn *)tc
            row:(int)row
{
    if (row == 0)
        return @"All Friends";
    else
        return [[groupArray objectAtIndex:row - 1] name];
}

// NSTableDataSource:
- (void)tableView:(NSTableView *)tv setObjectValue:(id)object
 forTableColumn:(NSTableColumn *)tc row:(int)row
{
    [(LJGroup *)[groupArray objectAtIndex:row - 1] setName:[object description]];
    [self reload];
}

// NSTableDataSource:
- (NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info
    proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation
{
    return (operation == NSTableViewDropOn && row > 0);
}

// NSTableDataSource:
- (BOOL)tableView:(NSTableView*)tv acceptDrop:(id <NSDraggingInfo>)info
              row:(int)row dropOperation:(NSTableViewDropOperation)operation
{
    NSString *type;
    NSArray *friendRows;
    int i;
    NSPasteboard *pb;
    LJGroup *group;

    if (row < 1) return NO;
    pb = [info draggingPasteboard];
    type = [pb availableTypeFromArray:[NSArray arrayWithObject:@"FriendRows"]];
    if (type == nil) return NO;
    friendRows = [pb propertyListForType:@"FriendRows"];
    group = [groupArray objectAtIndex:(row - 1)];
    for ( i = 0; i < [friendRows count]; i++ ) {
        int f = [[friendRows objectAtIndex:i] intValue];
        [group addFriend:[friends friendAtIndex:f]];
    }
    [friends reload];
    [window setDocumentEdited:YES];
    return YES;
}

// NSTableView Delegate:
- (BOOL)tableView:(NSTableView *)tv shouldEditTableColumn:(NSTableColumn *)tc
              row:(int)row
{
    return (row != 0); // Forbid editing of the "All Friends" group.
}

// NSTableView Delegate:
- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
    [friends reload];
}

// NSWindow Delegate:
- (void)windowDidBecomeMain:(NSNotification *)aNotification
{
    NS_DURING
        [self reload];
    NS_HANDLER
        NSLog(@"An exception was raised whilst downloading friends: %@",
              [localException reason]);
    NS_ENDHANDLER
}

// NSWindow Delegate:
- (BOOL)windowShouldClose:(id)sender
{
    LJAccount *account = [controller account];
    int r;

    NS_DURING
        NSLog(@"Upload Friends (%@) and Groups (%@)",
              [account uploadFriends] ? @"Changes Made" : @"No Changes",
              [account uploadGroups] ? @"Changes Made" : @"No Changes");
        [window setDocumentEdited:NO];
        [groupArray release];
        groupArray = nil;
        NS_VALUERETURN(YES, BOOL);
    NS_HANDLER
        r = NSRunAlertPanel(@"SimpleJournal Error", // *title,
                            @"While trying to save your changes to Friends & Groups, "
                            @"the following error occurred: %@", // *msg,
                            @"Leave Window Open", // *defaultButton,
                            @"Close Friends & Groups Window", // *alternateButton,
                            nil, // *otherButton,
                            [localException reason]); //...);
        return (r != NSAlertDefaultReturn);
    NS_ENDHANDLER
}

@end
