/*
 SimpleJournal: An example LiveJournal client written using LJKit.
 This program and its source code are placed in the public domain.
 */

#import "SimpleJournal.h"

#define LoginMenuItem 101
#define LogoutMenuItem 102
#define NewMenuItem 201
#define OpenMostRecentMenuItem 202
#define SaveMenuItem 203
#define RemoveMenuItem 204

@implementation SimpleJournal

/*
 Initialize the application controller.  We set up friends checking
 by registering this class to receive the appropriate notifications.
 The account object does the work of setting up a timer, doing the
 checking, and posting the notifications.
 */
- (id)init
{
    self = [super init];
    if (self) {
        NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
        [nc addObserver:self selector:@selector(friendsPageUpdated:)
                   name:LJFriendsPageUpdatedNotification object:nil];
        [nc addObserver:self selector:@selector(checkFriendsError:)
                   name:LJCheckFriendsErrorNotification object:nil];
        [nc addObserver:self selector:@selector(checkFriendsIntervalChanged:)
                   name:LJCheckFriendsIntervalChangedNotification object:nil];
    }
    return self;
}

/*
 Clean up after yourself!
 */
- (void)dealloc
{
    NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
    [nc removeObserver:self];
    [entry release];
    [account release];
    [super dealloc];
}

/*
 After the application launches we want to display the login sheet.
 At any time the client is not logged in, the sheet should be displayed.
 */
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [self beginLogin:nil];
}

/*
 A convenience method which returns the currently selected LJJournal
 from the pop up menu.
 */
- (LJJournal *)selectedJournal
{
    return [[journalPopUp selectedItem] representedObject];
}

/*
 This method is called by the main menu to enable or disable any menu
 items.  As you can see, we don't want every option available in every
 sitation.
 */
- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
    switch ([menuItem tag]) {
        case LogoutMenuItem:
            return !loggingIn;
        case NewMenuItem:
        case OpenMostRecentMenuItem:
            return (account != nil);
        case SaveMenuItem:
            return (entry != nil);
        case RemoveMenuItem:
            return ([entry itemID] != 0);
    }
    return YES;
}

/*
 This updates the Web menu on the menu bar, displaying "Not logged in"
 if we are not logged in, or the menu with web links.  LJKit sets up
 targets/actions internally so that selecting a menu item will cause
 the associated URL to open in the user's default browser.  All we have
 to do is display the menu in the interface.

 Note that we must remove the webMenuItem from the main menu before we
 can change the submenu.  If we just set a new submenu, duplicate Web
 menus appear in the main menu bar.  I don't know why this is, but it is.
 */
- (void)updateWebMenu
{
    NSMenu *mainMenu = [NSApp mainMenu];
    int index = [mainMenu indexOfItem:webMenuItem];
    [webMenuItem retain];
    [mainMenu removeItemAtIndex:index];
    [webMenuItem setSubmenu:(account == nil ?
                             notLoggedInMenu :
                             [account menu])];
    [mainMenu insertItem:webMenuItem atIndex:index];
    [webMenuItem release];
}

/*
 Begins login by displaying the login sheet attached to the main
 window.
 */
- (IBAction)beginLogin:(id)sender
{
    loggingIn = YES;
    [NSApp beginSheet:loginPanel modalForWindow:mainWindow
        modalDelegate:nil didEndSelector:NULL contextInfo:NULL];
}

/*
 Attempts to end login by creating an account object and logging into
 the LJ server.  If successful, we remove the sheet, set up the journal
 pop up menu, update the web menu, and start friends checking, change
 the main window title, and call newEntry:.  If we fail, we show the
 error in an alert panel and leave the login panel where it is.
 */
- (IBAction)endLogin:(id)sender
{
    NSString *username;
    NSString *urlString;

    NS_DURING
        [account release];
        username = [usernameField stringValue];
        account = [[LJAccount alloc] initWithUsername:username];
        urlString = [@"http://" stringByAppendingString:[hostField stringValue]];
        [[account server] setURL:[NSURL URLWithString:urlString]];
        [account loginWithPassword:[passwordField stringValue]];
        [NSApp endSheet:loginPanel];
        [loginPanel orderOut:nil];
        loggingIn = NO;
        // Setup Journal PopUp
        [journalPopUp setMenu:[account journalMenu]];
        [journalPopUp selectItemAtIndex:0];
        [journalPopUp setEnabled:YES];
        // Setup Web Menu
        [self updateWebMenu];
        // Setup CheckFriends
        [account setCheckFriendsInterval:60];
        [account startCheckingFriends];
        // Set Window Title
        [mainWindow setTitle:[account fullname]];
        [self newEntry:nil];
        // Show Login Message
        if ([account loginMessage]) {
            NSRunAlertPanel(@"Message from the Server", // *title,
                            [account loginMessage], // *msg,
                            nil, // *defaultButton,
                            nil, // *alternateButton,
                            nil); // *otherButton, ...);
        }
    NS_HANDLER
        NSRunAlertPanel(@"SimpleJournal Error", // *title,
                        [localException reason], // *msg,
                        nil, // *defaultButton,
                        nil, // *alternateButton,
                        nil); // *otherButton, ...);
        [account release];
        account = nil;
    NS_ENDHANDLER
}

/*
 Logout and clean up.  We clear the journal pop up menu, update the
 web menu, stop checking friends, restore the window tile, and release
 the account and entry objects we created.  Finally, we call beginLogin:
 to display the login sheet.
 */
- (IBAction)logout:(id)sender
{
    // Clear Journal PopUp
    [journalPopUp setEnabled:NO];
    [journalPopUp removeAllItems];
    // Clear Web Menu
    [self updateWebMenu];
    // Stop CheckFriends
    [account stopCheckingFriends];
    // Clean Up
    [mainWindow setTitle:@"SimpleJournal"];
    [entry release];
    entry = nil;
    [account release];
    account = nil;
    [self beginLogin:nil];
}

/*
 Creates a new entry object, and clears the main window's controls.
 */
- (IBAction)newEntry:(id)sender
{
    [entry release];
    entry = [[LJEntry alloc] init];
    [journalPopUp setEnabled:YES];
    [dateField setObjectValue:[entry date]];
    [subjectField setObjectValue:[entry subject]];
    [entryTextView setString:[entry content]];
}

/*
 Opens the most recent entry of the currently selected journal.
 The current entry is destroyed.  Note that even though SimpleJournal
 doesn't make use of many LJ features, such as security and metadata,
 the LJKit stores all of this information, so none of it is lost when
 we modify the entry object.

 Because this entry exists on the server (its itemID != 0) we disable
 the journal pop up menu so the user cannot change the journal it
 belongs to.
 */
- (IBAction)openMostRecentEntry:(id)sender
{
    LJEntry *newEntry;
    
    NS_DURING
        newEntry = [[[self selectedJournal] getMostRecentEntry] retain];
        if (newEntry) {
            [entry release];
            entry = [newEntry retain];
            [dateField setObjectValue:[entry date]];
            [subjectField setObjectValue:[entry subject]];
            [entryTextView setString:[entry content]];
            [journalPopUp setEnabled:NO];
        } else {
            NSRunAlertPanel(@"SimpleJournal",
                            @"No entry was returned by the server.  Sorry!",
                            nil, nil, nil);
        }
    NS_HANDLER
        NSRunAlertPanel(@"SimpleJournal Error", // *title,
                        [localException reason], // *msg,
                        nil, // *defaultButton,
                        nil, // *alternateButton,
                        nil); // *otherButton, ...);
    NS_ENDHANDLER
}

/*
 If the current entry exists on the server (has an itemID != 0) then
 this method will delete it from the server.  If successful,
 removeFromJournal will reset the itemID to 0.  Also, we enable the
 journal pop up menu, allowing the user to change which journal this
 entry belongs to now that it no longer exists on the server.
 */
- (IBAction)removeEntry:(id)sender
{
    NS_DURING
        [entry removeFromJournal];
        [journalPopUp setEnabled:YES];
    NS_HANDLER
        NSRunAlertPanel(@"SimpleJournal Error", // *title,
                        [localException reason], // *msg,
                        nil, // *defaultButton,
                        nil, // *alternateButton,
                        nil); // *otherButton, ...);
    NS_ENDHANDLER
}

/*
 Saves the journal entry to the server.  We only set the journal once,
 if the entry does not exist on the server (itemID == 0).  If the save
 is successful, we disable the journal pop up menu so that the user
 cannot change it.
 */
- (IBAction)saveEntry:(id)sender
{
    NS_DURING
        if ([entry itemID] == 0) {
            [entry setJournal:[self selectedJournal]];
        }
        [entry setDate:[dateField objectValue]];
        [entry setSubject:[subjectField stringValue]];
        [entry setContent:[entryTextView string]];
        [entry saveToJournal];
        [journalPopUp setEnabled:NO];
    NS_HANDLER
        NSRunAlertPanel(@"SimpleJournal Error", // *title,
                        [localException reason], // *msg,
                        nil, // *defaultButton,
                        nil, // *alternateButton,
                        nil); // *otherButton, ...);
    NS_ENDHANDLER
}

/*
 We redirect the Quit menu item to this method so that we can close the sheet
 before attempting to terminate.  Otherwise, the application will refuse to
 quit.
 */
- (IBAction)terminateApplication:(id)sender
{
    if (loggingIn) {
        [NSApp endSheet:loginPanel];
        [loginPanel orderOut:nil];
        loggingIn = NO;
    }
    [NSApp terminate:sender];
}

/*
 This method allows other controller classes to access the account object.
 */
- (LJAccount *)account
{
    return account;
}

/*
 This method is called when an account object posts
 LJFriendsPageUpdatedNotification, which is when one of the journals
 on this user's friends page is updated.

 In an application that supports multiple accounts concurrently, you
 can use [notification object] to return the account object that posted
 the notification.

 After posting this notification, the account object stops checking
 for friends updates.  You must start it again if you want to resume
 checking.
 */
- (void)friendsPageUpdated:(NSNotification *)notification
{
    int r =
    NSRunAlertPanel(@"SimpleJournal", // *title,
                    @"Your friends page has been updated!", // *msg,
                    @"View Page and Resume Checking", // *defaultButton,
                    @"Stop Checking", // *alternateButton,
                    nil); // *otherButton, ...);
    if (r == NSAlertDefaultReturn) {
        [[notification object] openFriendsPage];
        [[notification object] startCheckingFriends];
    }
}

/*
 This method is called when an account object posts
 LJCheckFriendsErrorNotification, which is when an error occurs during
 friends checking.

 After posting this notification, the account object stops checking
 for friends updates.  You must start it again if you want to resume
 checking.
 */
- (void)checkFriendsError:(NSNotification *)notification
{
    int r;
    NSString *errmsg;

    errmsg = [[[notification userInfo] objectForKey:@"LJException"] reason];
    r = NSRunAlertPanel(@"SimpleJournal", // *title,
                        @"While checking your friends page for updates, "
                        @"the following error occurred: %@", // *msg,
                        @"Resume Checking", // *defaultButton,
                        @"Stop Checking", // *alternateButton,
                        nil, // *otherButton,
                        errmsg); //...);
    if (r == NSAlertDefaultReturn) {
        [[notification object] startCheckingFriends];
    }
}

/*
 This method is called when an account object posts
 LJCheckFriendsIntervalChangedNotification, which is when the server
 orders the client to change the interval between update checks.
 Friends checking continues after this notification is posted, so there
 is no need to take action on the client's part.  All we do here
 is log a message to the console.
 */ 
- (void)checkFriendsIntervalChanged:(NSNotification *)notice
{
    NSLog(@"By order of the server, the check friends interval is "
          @"now %.3f seconds", [account checkFriendsInterval]);
}

@end
